@@ -56,6 +56,7 @@ SIGNAL_T      SigDone (int nonsense);
 
 void          CaptureAllWindows (ScreenInfo *scr);
 void          DoAutoexec( Bool restarting );
+void DeadPipe(int);
 
 Bool afterstep_parent_hints_func(Window parent, ASParentHints *dst );
 
@@ -80,7 +81,7 @@ LOCAL_DEBUG_OUT( "pid(%d),complete...", getpid() );
  *	main - start of afterstep
  ************************************************************************/
 int
-main (int argc, char **argv)
+main (int argc, char **argv, char **envp)
 {
     register int i ;
 	
@@ -92,16 +93,16 @@ main (int argc, char **argv)
 
 #ifdef LOCAL_DEBUG
 #if 0
-	LOCAL_DEBUG_OUT( "calibrating sleep_a_little : %s","" );
+	LOCAL_DEBUG_OUT( "calibrating sleep_a_millisec : %s","" );
 	for( i = 0 ; i < 500 ; ++i )
 		sleep_a_millisec( 10 );
-	LOCAL_DEBUG_OUT( "500 sliip_a_little(10) completed%s","" );
+	LOCAL_DEBUG_OUT( "500 sliip_a_millisec(10) completed%s","" );
 	for( i = 0 ; i < 50 ; ++i )
 		sleep_a_millisec( 100 );
-	LOCAL_DEBUG_OUT( "50 sliip_a_little(100) completed%s","" );
+	LOCAL_DEBUG_OUT( "50 sliip_a_millisec(100) completed%s","" );
 	for( i = 0 ; i < 10 ; ++i )
 		sleep_a_millisec( 300 );
-	LOCAL_DEBUG_OUT( "10 sliip_a_little(300) completed%s","" );
+	LOCAL_DEBUG_OUT( "10 sliip_a_millisec(300) completed%s","" );
 #endif
 #endif
 
@@ -110,6 +111,11 @@ main (int argc, char **argv)
 
 #ifdef DEBUG_TRACE_X
 	trace_window_id2name_hook = &window_id2name;
+#endif
+	set_DeadPipe_handler(DeadPipe);
+
+#if !HAVE_DECL_ENVIRON
+	override_environ( envp );
 #endif
     InitMyApp( CLASS_AFTERSTEP, argc, argv, NULL, NULL, 0);
 	for( i = 1 ; i< argc ; ++i)
@@ -121,6 +127,8 @@ main (int argc, char **argv)
 	    }
 	}
 
+	LinkAfterStepConfig();
+
     AfterStepState = MyArgs.flags ;
     clear_flags( AfterStepState, ASS_NormalOperation );
 	set_flags( AfterStepState, ASS_SuppressDeskBack );
@@ -141,14 +149,24 @@ main (int argc, char **argv)
     IgnoreSignal(SIGQUIT);
     IgnoreSignal(SIGTERM);
 
-    if( ConnectX( &Scr, AS_ROOT_EVENT_MASK ) < 0  )
+    if( ConnectX( ASDefaultScr, AS_ROOT_EVENT_MASK ) < 0  )
 	{
 		show_error( "Hostile X server encountered - unable to proceed :-(");
 		return 1;/* failed to accure window management selection - other wm is running */
 	}
 	XSetWindowBackground( dpy, Scr.Root, Scr.asv->black_pixel );
+	Scr.Look.desktop_animation_tint = get_random_tint_color();
+
     cover_desktop();
-    show_progress( "AfterStep is starting up ..." );
+	if( get_flags( AfterStepState, ASS_Restarting ))
+	{	
+		show_progress( "AfterStep v.%s is restarting ...", VERSION );
+		display_progress( True, "AfterStep v.%s is restarting ...", VERSION );
+	}else
+	{	
+    	show_progress( "AfterStep v.%s is starting up ...", VERSION );
+		display_progress( True, "AfterStep v.%s is starting up ...", VERSION );
+	}
 
 SHOW_CHECKPOINT;
 	InitSession();
@@ -170,6 +188,7 @@ SHOW_CHECKPOINT;
 	/*
      *  Lets init each and every screen separately :
      */
+	
     for (i = 0; i < Scr.NumberOfScreens; i++)
 	{
         show_progress( "Initializing screen %d ...", i );
@@ -187,7 +206,7 @@ SHOW_CHECKPOINT;
             }
         }else
         {
-            make_screen_envvars(&Scr);
+            make_screen_envvars(ASDefaultScr);
             putenv (Scr.rdisplay_string);
             putenv (Scr.display_string);
             if( is_output_level_under_threshold( OUTPUT_LEVEL_PROGRESS ) )
@@ -249,11 +268,11 @@ SHOW_CHECKPOINT;
 #ifndef DONT_GRAB_SERVER                    /* grabbed   !!!!!*/
 	grab_server();                		/* grabbed   !!!!!*/
 #endif										/* grabbed   !!!!!*/
-    init_screen_panframes(&Scr);            /* grabbed   !!!!!*/
+    init_screen_panframes(ASDefaultScr);            /* grabbed   !!!!!*/
     display_progress( True, "Capturing all windows ..." );
-    CaptureAllWindows (&Scr);               /* grabbed   !!!!!*/
+    CaptureAllWindows (ASDefaultScr);               /* grabbed   !!!!!*/
     display_progress( False, "Done." );
-    check_screen_panframes(&Scr);           /* grabbed   !!!!!*/
+    check_screen_panframes(ASDefaultScr);           /* grabbed   !!!!!*/
     ASSync( False );
 #ifndef DONT_GRAB_SERVER                    /* grabbed   !!!!!*/
 	ungrab_server();					/* UnGrabbed !!!!!*/
@@ -335,6 +354,7 @@ CreateManagementWindows()
     Scr.ServiceWin = create_visual_window (Scr.asv, Scr.Root, 0, 0, 1, 1, 0,
                                            InputOutput, CWEventMask | CWOverrideRedirect,
                                            &attr);
+	set_service_window_prop (Scr.wmprops, Scr.ServiceWin);
 	LOCAL_DEBUG_OUT( "Scr.ServiceWin = %lX;", Scr.ServiceWin );
     XMapRaised (dpy, Scr.ServiceWin);
     XSetInputFocus (dpy, Scr.ServiceWin, RevertToParent, CurrentTime);
@@ -436,7 +456,10 @@ CleanupScreen()
         if( Scr.RootBackground->pmap )
         {
             if( Scr.wmprops->root_pixmap == Scr.RootBackground->pmap )
+			{	
                 set_xrootpmap_id (Scr.wmprops, None );
+				set_as_background(Scr.wmprops, None );
+			}
             XFreePixmap( dpy, Scr.RootBackground->pmap );
             ASSync(False);
             LOCAL_DEBUG_OUT( "root pixmap with id %lX destroyed", Scr.RootBackground->pmap );
@@ -450,7 +473,7 @@ LOCAL_DEBUG_OUT("destroying font manager : %p", Scr.font_manager);
     destroy_font_manager( Scr.font_manager, False );
 
 LOCAL_DEBUG_OUT("destroying visual : %p", Scr.asv);
-    destroy_screen_gcs(&Scr);
+    destroy_screen_gcs(ASDefaultScr);
 	destroy_asvisual( Scr.asv, False );
 
 LOCAL_DEBUG_OUT("selecting input mask for Root window to 0 : %s","");
@@ -610,6 +633,7 @@ void
 Done (Bool restart, char *command )
 {
     int restart_screen = get_flags( AfterStepState, ASS_SingleScreen)?Scr.screen:-1;
+	Bool restart_self = False ; 
     char *local_command = NULL ;
 	{
 		static int already_dead = False ; 
@@ -617,13 +641,31 @@ Done (Bool restart, char *command )
 		already_dead = True ;
 	}
 
+#ifdef XSHMIMAGE
+	/* may not need to do that as server may still have some of the shared 
+	 * memory and work in it */
+	flush_shm_cache();
+#endif
 
 LOCAL_DEBUG_CALLER_OUT( "%s restart, cmd=\"%s\"", restart?"Do":"Don't", command?command:"");
 
     /* lets duplicate the string so we don't accidental;y delete it while closing self down */
     if( restart )
-        local_command = mystrdup(command?command:MyName);
+	{
+		if( command ) 
+		{
+			int my_name_len = strlen(MyName);
 
+			if( strncmp(command, MyName, my_name_len )==0 )
+				restart_self = (command[my_name_len] == ' '|| command[my_name_len] == '\0');
+			local_command = mystrdup(command);
+		}else 
+		{		  
+        	local_command = mystrdup(MyName);
+			restart_self = True ;
+	    }
+	}
+	LOCAL_DEBUG_OUT( "local_command = \"%s\", restart_self = %s", local_command, restart_self?"Yes":"No"); 
     set_flags( AfterStepState, ASS_Shutdown );
     if( restart )
         set_flags( AfterStepState, ASS_Restarting );
@@ -649,16 +691,15 @@ LOCAL_DEBUG_CALLER_OUT( "%s restart, cmd=\"%s\"", restart?"Do":"Don't", command?
     CleanupScreen();
     /* Really make sure that the connection is closed and cleared! */
     XSync (dpy, 0);
+
 #ifdef XSHMIMAGE
-	/* may not need to do that as server may still have some of the shared 
-	 * memory and work in it */
 	flush_shm_cache();
 #endif
-
 	if (restart)
 	{
+		set_flags( MyArgs.flags, ASS_Restarting );
         spawn_child( local_command, -1, restart_screen,
-                     None, C_NO_CONTEXT, False, True, NULL );
+                     None, C_NO_CONTEXT, False, restart_self, NULL );
     } else
 	{
 	    XCloseDisplay (dpy);
@@ -672,21 +713,25 @@ LOCAL_DEBUG_CALLER_OUT( "%s restart, cmd=\"%s\"", restart?"Do":"Don't", command?
     	/* pixmap references */
     	build_xpm_colormap (NULL);
 
-#ifdef DEBUG_ALLOCS
         restack_window_list(INVALID_DESK, True);
         clientprops_cleanup ();
         wmprops_cleanup ();
         free_func_hash();
         purge_asimage_registry();
-        flush_ashash_memory_pool();
-        flush_asbidirlist_memory_pool();
-        free( MyArgs.saved_argv );
 		asxml_var_cleanup();
 		custom_color_cleanup(); 
 		flush_default_asstorage();
+        flush_ashash_memory_pool();
+        flush_asbidirlist_memory_pool();
+        free( MyArgs.saved_argv );
+#ifdef DEBUG_ALLOCS
         print_unfreed_mem ();
 #endif /*DEBUG_ALLOCS */
-    }
+#ifdef XSHMIMAGE
+		flush_shm_cache();
+#endif
+    
+	}
     exit(0);
 }
 
